"""
A module to estimate the autocorrelation time of time-series data extremely quickly

Original C++ routine written by Jonathan Goodman (NYU)
Python wrapper written by Dan Foreman-Mackey (NYU)

"""

from acor import *

