/*
 * This is a very slightly re-factored version of code by Jonathan Goodman
 * For the original code, see:
 *  http://www.math.nyu.edu/faculty/goodman/software/acor/index.html
 * Ported for use in Python with permission from the original author.
 */

int acor(double *mean, double *sigma, double *tau, double X[], int L, int maxlag);

